#version 330 core
#extension GL_ARB_explicit_attrib_location: enable

// Per vertex attribute
layout(location = 0) in vec3 vertexPosition;	//Per vertex
layout(location = 1) in vec4 rgbaBase;			// Per vertex
layout(location = 2) in int flags;			// Per vertex

//layout(location = 3) in float cloudOffsetY;			// Per instance

// custom ints
layout(location = 3) in ivec2 cloudTileOffset;	// Per instance. 

// custom bytes
layout(location = 4) in vec4 cloudDensityBySide;		// Per instance
layout(location = 5) in float upDownDensity;	// Per instance
layout(location = 6) in float cloudBrightness;	// Per instance



// Per drawcall attribute
uniform vec3 sunPosition;
uniform vec3 sunColor;
uniform float dayLight;
uniform vec3 windOffset;
uniform vec4 rgbaFogIn;
uniform vec3 playerPos;
uniform float fogMinIn;
uniform float fogDensityIn;
uniform float globalCloudBrightness;
uniform int cloudTileSize;
uniform float cloudsLength;

uniform mat4 projectionMatrix;
uniform mat4 modelViewMatrix;

out vec4 vertexColor;

#include colorutil.ash
#include shadowcoords.vsh
#include fogandlight.vsh


float getCloudFogLevel(float flatFogdepth, float normalFogDepth, float fogMin, float fogDensity, float worldPosY) {
	float nearFogRemoval =  -clamp((2.5 / pow(1.25, flatFogdepth / 1)), 0, 0.85);  // (for removing fog very close to the player);
	
	float val = max(
		fogMin + 1 - 1 / exp((7*normalFogDepth+10) * fogDensity + max(-flatFogDensity * normalFogDepth * (flatFogStart - 5) / 8, 0)) + nearFogRemoval, 
		1 - 1 / exp((80 + worldPosY - flatFogStart) * flatFogDensity) + nearFogRemoval
	);

	return clamp(val, 0, 1);
}

void main()
{
	float cloudDensity = flags >= 4 ? upDownDensity : cloudDensityBySide[flags];

	// Dense clouds are darker
	float darkener = max(0, cloudDensity - 0.85f);
	
	vec3 vexPos = vertexPosition + vec3(cloudTileOffset.x * cloudTileSize, 0, cloudTileOffset.y * cloudTileSize);
	
	// Dense clouds are taller
	if (vexPos.y > 0) vexPos.y += cloudDensity * 12;
	
	// Add "earth" curvature
	vexPos.y -= max(length(vexPos.xz + windOffset.xz) / 10 - 50, 0);
	
	
	vec3 Sn = normalize(sunPosition);
	
	gl_Position = projectionMatrix * modelViewMatrix * vec4(vexPos, 1.0);
	float fogAmount = getCloudFogLevel(gl_Position.w, gl_Position.w * 0.05 - 10, fogMinIn, 0.005 + fogDensityIn, vexPos.y);
		
	vertexColor = vec4(rgbaBase.rgb - darkener, cloudDensity);
	vertexColor.rgb *= rgbaFogIn.rgb;

	// Blend to fog color 
	vertexColor.rgb = 
		mix(
			vertexColor.rgb * cloudBrightness * globalCloudBrightness,
			rgbaFogIn.rgb,
			fogAmount
		);
	
	
	// Fade out distant clouds
	float distToEdge = cloudsLength/2 - length(vexPos);
	vertexColor.a *= max(0, min(1, distToEdge/(cloudsLength * 0.4) - 0));
	
	
}
